@@ -8,32 +8,94 @@ import math
 import os
 import re
 import shutil
+import subprocess
 import sys
 import tarfile
+import tempfile
 import zipfile
 from urllib.request import Request, urlopen, urlsplit
 
 from appdirs import user_data_dir
 from diskcache import Cache, Timeout as CacheTimeout
-from pkg_resources.extern.packaging.version import Version
 from tqdm import tqdm
 
 
 __author__ = "mondeja"
 __description__ = "Multiplatform Blender portable release downloader script."
 __title__ = "blender-downloader"
-__version__ = "0.0.19"
+__version__ = "0.0.23"
 
 QUIET = False
 
+TEMPDIR = os.path.join(tempfile.gettempdir(), "blender-downloader")
+DATA_DIR = user_data_dir(appname=__title__, appauthor=__author__, version=__version__)
+CACHE = Cache(DATA_DIR)
+
 SCRIPT_NEW_ISSUE_URL = f"https://github.com/{__author__}/{__title__}/issues/new"
 BLENDER_MANUAL_VERSIONS_URL = "https://docs.blender.org/PROD/versions.json"
+BLENDER_DAILY_BUILDS_URL = "https://builder.blender.org/download/daily/"
 MINIMUM_VERSION_SUPPPORTED = "2.57"
-SUPPORTED_FILETYPES_EXTRACTION = [".bz2", ".gz", ".xz", ".zip", ".dmg"]
 NIGHLY_RELEASES_CACHE_EXPIRATION = 60 * 60 * 24  # 1 day
-CACHE = Cache(
-    user_data_dir(appname=__title__, appauthor=__author__, version=__version__)
-)
+
+
+def removesuffix(string, suffix):  # polyfill for Python < 3.9
+    if string.endswith(suffix):
+        return string[: -len(suffix)]
+    return string
+
+
+def controlled_full_splitext(path, possible_extensions):
+    extension = None
+    for ext in possible_extensions:
+        if path.endswith(ext):
+            extension = ext
+            break
+    return extension
+
+
+class BlenderVersion:
+    """Blender versions object for easy comparations support."""
+
+    def __init__(self, raw):
+        self.raw = raw
+        self.version_info = []
+        self.alphas = []
+
+        for num_or_alpha in raw.split("."):
+            num = ""
+            for char in num_or_alpha:
+                if char.isdigit() and not self.alphas:
+                    num += char
+                else:
+                    self.alphas.append(char)
+            if num:
+                self.version_info.append(int(num))
+        for alpha in self.alphas:
+            if alpha.isdigit():  # digits as alphas like '3' in '2.80rc3'
+                self.version_info.append(int(alpha))
+            else:
+                self.version_info.append(ord(alpha))
+
+    def __lt__(self, other):
+        return self.version_info < other.version_info
+
+    def __le__(self, other):
+        return self.version_info <= other.version_info
+
+    def __gt__(self, other):
+        return self.version_info > other.version_info
+
+    def __ge__(self, other):
+        return self.version_info >= other.version_info
+
+    def __eq__(self, other):
+        return self.version_info == other.version_info
+
+    def __str__(self):
+        return self.raw
+
+    def __repr__(self):
+        return f'BlenderVersion("{self.raw}")'
 
 
 class BlenderVersionNotFound(RuntimeError):
@@ -57,6 +119,35 @@ def GET(url, expire=259200, use_cache=True):  # 3 days for expiration
     return response.decode("utf-8")
 
 
+def get_toplevel_dirnames_from_paths(paths):
+    """Extracts the names of the directories in the top level
+    from a set of paths constituting multiple directory trees.
+    """
+    toplevel_dirnames = []
+    for path in paths:
+        # not optimal implementation, but crossplatform for sure
+        parent, _ = os.path.split(path)
+        if parent:  # is not file, when is file parent == ''
+            while parent:
+                previous_parent = parent
+                parent, _ = os.path.split(previous_parent)
+            if previous_parent not in toplevel_dirnames:
+                toplevel_dirnames.append(previous_parent)
+    return toplevel_dirnames
+
+
+def clean_other_versions_cache():
+    """Remove the cache for other versions of blender-downloader
+    installed. Only has sense to execute a single version, the
+    latest.
+    """
+    caches_dir = os.path.abspath(os.path.dirname(DATA_DIR))
+    for dirname in os.listdir(caches_dir):
+        if dirname != __version__:
+            dirpath = os.path.join(caches_dir, dirname)
+            shutil.rmtree(dirpath)
+
+
 def build_parser():
     parser = argparse.ArgumentParser(description=__description__)
     parser.add_argument(
@@ -103,6 +194,7 @@ def build_parser():
         " This is not supported for MacOS releases using Python3.6.",
     )
     parser.add_argument(
+        "-r",
         "--remove-compressed",
         dest="remove_compressed",
         action="store_true",
@@ -110,6 +202,7 @@ def build_parser():
         " takes effect if '--extract' option is passed.",
     )
     parser.add_argument(
+        "-b",
         "--print-blender-executable",
         dest="print_blender_executable",
         action="store_true",
@@ -118,6 +211,7 @@ def build_parser():
         " along with '--extract' or won't take effect.",
     )
     parser.add_argument(
+        "-p",
         "--print-python-executable",
         dest="print_python_executable",
         action="store_true",
@@ -127,6 +221,7 @@ def build_parser():
         " take effect.",
     )
     parser.add_argument(
+        "-o",
         "--os",
         "--operative-system",
         dest="operative_system",
@@ -145,6 +240,7 @@ def build_parser():
         " latest release with support operative systems with 32 bits.",
     )
     parser.add_argument(
+        "-a",
         "--arch",
         dest="arch",
         default=None,  # depending on availability between versions
@@ -165,6 +261,7 @@ def build_parser():
         " versions. You can pass an optional maximum number of versions to list.",
     )
     parser.add_argument(
+        "-n",
         "--no-cache",
         "--nocache",
         dest="use_cache",
@@ -172,6 +269,7 @@ def build_parser():
         help="Don't use cache requesting Blender repositories.",
     )
     parser.add_argument(
+        "-c",
         "--invalidate-cache",
         "--clear-cache",
         dest="clear_cache",
@@ -229,7 +327,7 @@ def parse_args(args):
                 opts.blender_version,
                 use_cache=opts.use_cache,
             )
-        elif Version(opts.blender_version) >= Version("2.83"):
+        elif BlenderVersion(opts.blender_version) >= BlenderVersion("2.83"):
             opts.blender_version = normalize_version(opts.blender_version)
     else:
         opts.blender_version = None
@@ -251,7 +349,9 @@ def parse_args(args):
         sys.exit(1)
 
     # assert compatible bits
-    if opts.bits == 32 and Version(opts.blender_version) > Version("2.80"):
+    if opts.bits == 32 and BlenderVersion(opts.blender_version) > BlenderVersion(
+        "2.80"
+    ):
         sys.stderr.write(
             "The latest Blender version with support for 32 bits systems is"
             " v2.80. Please, specify a more recent version of Blender.\n"
@@ -263,10 +363,38 @@ def parse_args(args):
 
     if opts.use_cache:
         CACHE.expire()  # remove expired items from cache
+        clean_other_versions_cache()
 
     return opts
 
 
+def guess_stable_version_number_from_daily_builds_page(use_cache=True):
+    """Try to get the stable Blender version from the page displayed at
+    https://builder.blender.org/download/daily/
+
+    This method has been proved unstable in the past so must not be used
+    as the unique reliable source.
+
+    use_cache : bool
+      Use cache requesting Blender versions from Blender's builder page.
+    """
+    res = GET(
+        BLENDER_DAILY_BUILDS_URL,
+        expire=NIGHLY_RELEASES_CACHE_EXPIRATION,
+        use_cache=use_cache,
+    )
+
+    stable_Version = None
+    for line in res.split("<"):
+        line = line.lower()
+        if "stable" in line:
+            stable_version_match = re.search(r"blender-([^-]+)", line)
+            if stable_version_match is not None:
+                stable_Version = BlenderVersion(stable_version_match.group(1))
+                break
+    return stable_Version
+
+
 def discover_version_number_by_identifier(identifier, use_cache=True):
     """Discover a Blender version number given an identifier.
 
@@ -278,38 +406,53 @@ def discover_version_number_by_identifier(identifier, use_cache=True):
     use_cache : bool
       Use cache requesting Blender versions from manual.
     """
-    res = GET(
-        BLENDER_MANUAL_VERSIONS_URL,
-        expire=NIGHLY_RELEASES_CACHE_EXPIRATION,
-        use_cache=use_cache,
-    )
-    versions_json = json.loads(res)
-
     if identifier == "stable":
-        latest_Version = None
-        for minor_version, version_data in versions_json.items():
-            if "dev" in version_data:
-                continue
-
-            minor_Version = Version(minor_version)
-            if latest_Version is None or minor_Version > latest_Version:
-                latest_Version = minor_Version
+        # try to get stable version from Blender's builder page
+        latest_Version = guess_stable_version_number_from_daily_builds_page(
+            use_cache=use_cache,
+        )
 
+        # fallback to versions JSON in documentation
         if latest_Version is None:
-            sys.stderr.write(
-                "Error trying to retrieve the stable release from Blender"
-                " repositories. Please, submit an issue to"
-                " {SCRIPT_NEW_ISSUE_URL}.\n"
+            versions_json = json.loads(
+                GET(
+                    BLENDER_MANUAL_VERSIONS_URL,
+                    expire=NIGHLY_RELEASES_CACHE_EXPIRATION,
+                    use_cache=use_cache,
+                )
             )
-            sys.exit(1)
+
+            latest_Version = None
+            for minor_version, version_data in versions_json.items():
+                if "dev" in version_data:
+                    continue
+
+                minor_Version = BlenderVersion(minor_version)
+                if latest_Version is None or minor_Version > latest_Version:
+                    latest_Version = minor_Version
+
+            if latest_Version is None:
+                sys.stderr.write(
+                    "Error trying to retrieve the stable release from Blender"
+                    " repositories. Please, submit a report to"
+                    " {SCRIPT_NEW_ISSUE_URL}\n"
+                )
+                sys.exit(1)
     elif identifier in {"lts", "nightly", "daily"}:
         expected_substr_in_data = "lts" if identifier == "lts" else "dev"
         latest_Version = None
+        versions_json = json.loads(
+            GET(
+                BLENDER_MANUAL_VERSIONS_URL,
+                expire=NIGHLY_RELEASES_CACHE_EXPIRATION,
+                use_cache=use_cache,
+            )
+        )
 
         for minor_version, version_data in versions_json.items():
             if expected_substr_in_data not in version_data.lower():
                 continue
-            minor_Version = Version(minor_version)
+            minor_Version = BlenderVersion(minor_version)
             if latest_Version is None or minor_Version > latest_Version:
                 latest_Version = minor_Version
     else:
@@ -326,19 +469,19 @@ def _build_download_repo_expected_os_identifier(
     major_minor_blender_Version,
 ):
     if operative_system == "macos":
-        if major_minor_blender_Version < Version("2.61"):
+        if major_minor_blender_Version < BlenderVersion("2.61"):
             expected_os_identifier = "OSX"
-        elif major_minor_blender_Version < Version("2.65"):
+        elif major_minor_blender_Version < BlenderVersion("2.65"):
             expected_os_identifier = "release-OSX"
-        elif major_minor_blender_Version < Version("2.79"):
+        elif major_minor_blender_Version < BlenderVersion("2.79"):
             # previous to v2.79, macOS was identified by "OSX"
             expected_os_identifier = "OSX"
         else:
             expected_os_identifier = "mac"  # some macOS, other macos
     elif operative_system == "windows":
-        if major_minor_blender_Version < Version("2.61"):
+        if major_minor_blender_Version < BlenderVersion("2.61"):
             expected_os_identifier = "windows"
-        elif major_minor_blender_Version < Version("2.66"):
+        elif major_minor_blender_Version < BlenderVersion("2.66"):
             expected_os_identifier = "release-windows"
         else:
             expected_os_identifier = operative_system
@@ -357,7 +500,7 @@ def _build_download_repo_release_file_validator(
 
         def valid_release_file(filename):
             # without 32 bits support
-            if major_minor_blender_Version > Version("2.80"):
+            if major_minor_blender_Version > BlenderVersion("2.80"):
                 if not filename.endswith(".zip"):
                     return False
             else:
@@ -367,14 +510,14 @@ def _build_download_repo_release_file_validator(
 
     elif operative_system == "linux":
         # before v2.82, Linux releases was distributed in .tar.bz2
-        if major_minor_blender_Version < Version("2.82"):
+        if major_minor_blender_Version < BlenderVersion("2.82"):
             compressed_ext = ".tar.bz2"
         else:
             compressed_ext = ".tar.xz"
 
         def valid_release_file(filename):
             # without 32 bits support
-            if major_minor_blender_Version > Version("2.80"):
+            if major_minor_blender_Version > BlenderVersion("2.80"):
                 if not filename.endswith(compressed_ext):
                     return False
             else:
@@ -384,21 +527,21 @@ def _build_download_repo_release_file_validator(
             return True
 
     else:  # operative_system == "macos"
-        if major_minor_blender_Version < Version("2.79"):
+        if major_minor_blender_Version < BlenderVersion("2.79"):
             # previous to v2.79, macos release was distributed in .zip
             compressed_ext = ".zip"
-        elif major_minor_blender_Version == Version("2.79"):
+        elif major_minor_blender_Version == BlenderVersion("2.79"):
             compressed_ext = ".tar.gz"
         else:
             compressed_ext = ".dmg"
 
         def valid_release_file(filename):
-            if major_minor_blender_Version < Version("2.72"):
+            if major_minor_blender_Version < BlenderVersion("2.72"):
                 # previous to v2.72, macos release supported 32 bits
                 bits_id = "x86_64" if (bits == 64 or arch == "x86_64") else "i386"
                 if not filename.endswith(f"{bits_id}{compressed_ext}"):
                     return False
-            elif major_minor_blender_Version >= Version("2.93"):
+            elif major_minor_blender_Version >= BlenderVersion("2.93"):
                 # from v2.93, Blender supports arm64 builds for macOS
                 if arch in ["x64", "arm64"]:
                     if not filename.endswith(f"{arch}{compressed_ext}"):
@@ -441,9 +584,11 @@ def get_legacy_release_download_url(
     major_minor_blender_version = re.sub(
         r"[a-zA-Z]", "", ".".join(blender_version.split(".")[:2])
     )
-    major_minor_blender_Version = Version(major_minor_blender_version)
+    major_minor_blender_Version = BlenderVersion(major_minor_blender_version)
 
-    if Version(major_minor_blender_version) < Version(MINIMUM_VERSION_SUPPPORTED):
+    if BlenderVersion(major_minor_blender_version) < BlenderVersion(
+        MINIMUM_VERSION_SUPPPORTED
+    ):
         sys.stderr.write(
             "The minimum version supported by blender-downloader is"
             f" {MINIMUM_VERSION_SUPPPORTED}.\n"
@@ -578,37 +723,46 @@ def download_release(download_url, output_directory, quiet=False):
     output_directory : str
       Path to the directory in which the downloaded file will be stored.
     """
-    # get filename of downloaded file (maybe a zip, maybe a dmg...)
-    output_filename = os.path.basename(urlsplit(download_url).path)
-    output_filepath = os.path.join(output_directory, output_filename)
+    try:
+        # get filename of downloaded file (maybe a zip, maybe a dmg...)
+        output_filename = os.path.basename(urlsplit(download_url).path)
+        output_filepath = os.path.join(output_directory, output_filename)
 
-    if os.path.isfile(output_filepath):
-        sys.stderr.write(
-            f"There is already a file named as '{output_filename}' in the"
-            " directory in which Blender will be downloaded.\nPlease, remove"
-            " the file before execute blender-downloader.\n"
+        if os.path.isfile(output_filepath):
+            sys.stderr.write(
+                f"There is already a file named as '{output_filename}' in the"
+                " directory in which Blender will be downloaded.\nPlease, remove"
+                " the file before execute blender-downloader.\n"
+            )
+            sys.exit(1)
+
+        # create temporal blender-downloader directory if not exists to store
+        # extracted files
+        if not os.path.isdir(TEMPDIR):
+            os.mkdir(TEMPDIR)
+
+        tmp_output_filepath = os.path.join(TEMPDIR, output_filename)
+
+        chunksize = 8192
+        downloaded_size = chunksize
+        res = urlopen(Request(download_url))
+        total_size_bits = int(res.info()["Content-Length"])
+
+        progress_bar_kwargs = dict(
+            total=total_size_bits,
+            unit="B",
+            desc=f"Downloading '{output_filename}'",
+            unit_scale=True,
+            unit_divisor=1000,
+            miniters=1,
+            disable=quiet,
+            initial=chunksize,  # first chunk is written before entering while
         )
-        sys.exit(1)
-
-    chunksize = 8192
-    downloaded_size = chunksize
-    res = urlopen(Request(download_url))
-    total_size_bits = int(res.info()["Content-Length"])
-
-    progress_bar_kwargs = dict(
-        total=total_size_bits,
-        unit="B",
-        desc=f"Downloading '{output_filename}'",
-        unit_scale=True,
-        unit_divisor=1000,
-        miniters=1,
-        disable=quiet,
-    )
-    with tqdm(**progress_bar_kwargs) as progress_bar:
-        data = res.read(chunksize)
-        with open(output_filepath, "wb") as f:
+        with tqdm(**progress_bar_kwargs) as progress_bar, open(
+            tmp_output_filepath, "wb"
+        ) as f:
+            data = res.read(chunksize)
             f.write(data)
-            progress_bar.update(chunksize)
             while data:
                 data = res.read(chunksize)
                 f.write(data)
@@ -616,6 +770,19 @@ def download_release(download_url, output_directory, quiet=False):
                 downloaded_size += chunksize
                 if downloaded_size >= total_size_bits:
                     break
+    except KeyboardInterrupt:
+        sys.stderr.write("Download interrupted\n")
+        if os.path.isfile(tmp_output_filepath):
+            os.remove(tmp_output_filepath)
+        sys.exit(1)
+
+    # move from temporal directory to the real output path
+    #
+    # use `shutil.move` instead of `os.rename` to be able to
+    # move files between different hard drives, see:
+    # https://stackoverflow.com/a/21116654/9167585
+    shutil.move(tmp_output_filepath, output_filepath)
+
     return output_filepath
 
 
@@ -634,97 +801,188 @@ def extract_release(zipped_filepath, quiet=False):
     Returns:
       str: Blender executable file path.
     """
-    zipped_filename = os.path.basename(zipped_filepath)
-    output_directory = os.path.abspath(os.path.dirname(zipped_filepath))
-    extension = os.path.splitext(zipped_filepath)[1]
+    try:
+        zipped_filename = os.path.basename(zipped_filepath)
+        output_directory = os.path.abspath(os.path.dirname(zipped_filepath))
+        short_extension = os.path.splitext(zipped_filepath)[-1]
 
-    if extension not in SUPPORTED_FILETYPES_EXTRACTION:
-        sys.stderr.write(
-            f"Blender compressed release file '{zipped_filename}' extraction"
-            " is not supported by blender-downloader.\n"
-        )
-        sys.exit(1)
+        # filepath of the extracted directory, don't confuse it with
+        # `output_directory`, that is the directory where the file to extract
+        # is located
+        extracted_directory_path = None
 
-    # filepath of the extracted directory, don't confuse it with
-    # `output_directory`, that is the directory where the file to extract
-    # is located
-    extracted_directory_filepath = None
+        if short_extension == ".zip":
+            if not quiet:
+                sys.stderr.write(f"Decompressing '{zipped_filename}'...\n")
 
-    if extension == ".zip":
-        if not quiet:
-            sys.stderr.write(f"Decompressing '{zipped_filename}'...\n")
+            with zipfile.ZipFile(zipped_filepath, "r") as f:
+                namelist = f.namelist()
+                namelist_length = len(namelist)
 
-        with zipfile.ZipFile(zipped_filepath, "r") as f:
-            namelist = f.namelist()
-            extracted_directory_filepath = os.path.join(
-                output_directory,
-                namelist[0].split(os.sep)[0],
-            )
-            progress_bar_kwargs = dict(
-                total=len(namelist),
-                desc=f"Extracting '{zipped_filename}'",
-                iterable=namelist,
-                disable=quiet,
-            )
-            for file in tqdm(**progress_bar_kwargs):
-                f.extract(member=file, path=output_directory)
-    elif extension in [".bz2", ".gz", ".xz"]:
-        if not quiet:
-            sys.stderr.write(f"Decompressing '{zipped_filename}'...\n")
+                # ensure that Blender is extracted in a top level directory
+                #
+                # MacOS versions previous to 2.79 are released in a ZIP file
+                # with multiple directories and files in the root, which results
+                # in a messy extraction with a lot of files in the current
+                # working directory
+                root_dirnames = get_toplevel_dirnames_from_paths(namelist)
 
-        with tarfile.open(zipped_filepath, "r") as f:
-            members = f.getmembers()
+                # `not root_dirnames` when only files are found in the ZIP
+                if len(root_dirnames) > 1 or not root_dirnames:
+                    output_directory = os.path.join(output_directory, "Blender")
 
-            extracted_directory_filepath = os.path.join(
-                output_directory,
-                members[0].name.split(os.sep)[0],
-            )
-            progress_bar_kwargs = dict(
-                total=len(members),
-                desc=f"Extracting '{zipped_filename}'",
-                iterable=members,
-                disable=quiet,
-            )
-            for file in tqdm(**progress_bar_kwargs):
-                f.extract(member=file, path=output_directory)
-    else:  # extension == ".dmg":
-        running_os = get_running_os()
-        if running_os != "macos":
+                    # don't overwrite existing non empty directory extracting
+                    if os.path.isdir(output_directory) and os.listdir(output_directory):
+                        sys.stderr.write(
+                            f"The directory '{output_directory}' where the files will"
+                            " be extracted already exists and is not empty. Extraction"
+                            " skipped.\n"
+                        )
+                        sys.exit(1)
+
+                    extracted_directory_path = output_directory
+                else:
+                    extracted_directory_path = os.path.join(
+                        output_directory, removesuffix(zipped_filename, ".zip")
+                    )
+
+                progress_bar_kwargs = dict(
+                    total=namelist_length,
+                    desc=f"Extracting '{zipped_filename}'",
+                    iterable=namelist,
+                    disable=quiet,
+                )
+                for file in tqdm(**progress_bar_kwargs):
+                    f.extract(member=file, path=output_directory)
+
+        elif short_extension in [".bz2", ".gz", ".xz"]:
+            if not quiet:
+                sys.stderr.write(f"Decompressing '{zipped_filename}'...\n")
+
+            with tarfile.open(zipped_filepath, "r") as f:
+                files = f.getmembers()
+                files_length = len(files)
+                paths = [file.name for file in files]
+
+                root_dirnames = get_toplevel_dirnames_from_paths(paths)
+
+                if len(root_dirnames) > 1 or not root_dirnames:
+                    output_directory = os.path.join(output_directory, "Blender")
+                    extracted_directory_path = output_directory
+                else:
+                    long_extension = controlled_full_splitext(
+                        zipped_filename,
+                        [".tar.gz", ".tar.xz", ".tar.bz2", ".gz", ".xz", ".bz2"],
+                    )
+                    extracted_directory_path = os.path.join(
+                        output_directory, removesuffix(zipped_filename, long_extension)
+                    )
+
+                progress_bar_kwargs = dict(
+                    total=files_length,
+                    desc=f"Extracting '{zipped_filename}'",
+                    iterable=files,
+                    disable=quiet,
+                )
+                for file in tqdm(**progress_bar_kwargs):
+                    f.extract(member=file, path=output_directory)
+
+        elif short_extension == ".dmg":
+            running_os = get_running_os()
+            if running_os != "macos":
+                # we are not in MacOS, so we need the binaries dmg2img and 7z
+                # to decompress the `.dmg` file downloaded
+                dmg2img, sevenz = shutil.which("dmg2img"), shutil.which("7z")
+                if dmg2img is None or sevenz is None:
+                    required_programs = []
+                    if dmg2img is None:
+                        required_programs.append("'dmg2img'")
+                    if sevenz is None:
+                        required_programs.append("'7z'")
+                    plural_suffix = "s" if len(required_programs) > 1 else ""
+                    sys.stderr.write(
+                        f"You need to install the program{plural_suffix}"
+                        f" {' and '.join(required_programs)} to extract the"
+                        f" DMG Blender release located at {zipped_filepath}"
+                        f" inside a {running_os.capitalize()} platform.\n"
+                    )
+                    sys.exit(1)
+
+                img_filepath = removesuffix(zipped_filepath, "dmg") + "img"
+                if os.path.isfile(img_filepath):
+                    os.remove(img_filepath)
+
+                dmg2img_proc = subprocess.Popen(
+                    ["dmg2img", zipped_filepath],
+                    stderr=sys.stderr,
+                    stdout=sys.stdout,
+                    env=os.environ,
+                )
+                dmg2img_proc.communicate()
+                if dmg2img_proc.returncode != 0:
+                    sys.exit(dmg2img_proc.returncode)
+
+                seven7_proc = subprocess.Popen(
+                    ["7z", "x", img_filepath],
+                    stderr=sys.stderr,
+                    stdout=sys.stdout,
+                    env=os.environ,
+                )
+                seven7_proc.communicate()
+                if seven7_proc.returncode != 0:
+                    sys.exit(seven7_proc.returncode)
+
+                extracted_directory_path = os.path.join(output_directory, "Blender")
+            else:
+                # we are inside MacOS, use the DMG CLI utility included in
+                # the system through the dmglib Python wrapper
+                extracted_directory_path = os.path.join(
+                    output_directory,
+                    removesuffix(os.path.basename(zipped_filepath), ".dmg"),
+                )
+
+                import dmglib
+
+                with dmglib.attachedDiskImage(zipped_filepath) as mounted_dmg:
+                    contents_parent_dirpath = None
+                    for dirpath, _, _ in os.walk(mounted_dmg[0]):
+                        if (
+                            os.path.basename(dirpath) == "Contents"
+                            and "blender.app" in dirpath.lower()
+                        ):
+                            contents_parent_dirpath = os.path.abspath(
+                                os.path.dirname(dirpath),
+                            )
+                            break
+                    shutil.copytree(
+                        contents_parent_dirpath,
+                        extracted_directory_path,
+                    )
+        else:
+            if "-e" in sys.argv and "--extract" not in sys.argv:
+                extract_option = "-e"
+            elif "-e" not in sys.argv and "--extract" in sys.argv:
+                extract_option = "--extract"
+            else:
+                extract_option = "-e/--extract"
             sys.stderr.write(
-                "blender-downloader can't mount MacOS '.dmg' image files like"
-                f" '{extracted_directory_filepath}' in"
-                f" {running_os.capitalize()}, so you should install Blender"
-                " manually.\n"
+                f"File extension '{short_extension}' extraction not"
+                " supported by blender-downloader's command line option"
+                f" '{extract_option}'.\n"
             )
             sys.exit(1)
+    except KeyboardInterrupt:
+        sys.stderr.write("Extraction interrupted\n")
+        # remove the directory being created, if exists
+        if os.path.isdir(extracted_directory_path):
+            shutil.rmtree(extracted_directory_path)
+        sys.exit(1)
 
-        extracted_directory_filepath = os.path.join(
-            output_directory, os.path.basename(zipped_filepath).rstrip(".dmg")
-        )
-
-        import dmglib
-
-        with dmglib.attachedDiskImage(zipped_filepath) as mounted_dmg:
-            contents_parent_dirpath = None
-            for dirpath, _, _ in os.walk(mounted_dmg[0]):
-                if (
-                    os.path.basename(dirpath) == "Contents"
-                    and "blender.app" in dirpath.lower()
-                ):
-                    contents_parent_dirpath = os.path.abspath(
-                        os.path.dirname(dirpath),
-                    )
-                    break
-            shutil.copytree(
-                contents_parent_dirpath,
-                extracted_directory_filepath,
-            )
-
-    return extracted_directory_filepath
+    return extracted_directory_path
 
 
 def print_executables(
-    extracted_directory_filepath,
+    extracted_directory_path,
     operative_system,
     print_blender_executable,
     print_python_executable,
@@ -736,7 +994,7 @@ def print_executables(
     Parameters
     ----------
 
-    extracted_directory_filepath : bool
+    extracted_directory_path : bool
       Blender release directory.
 
     operative_system : str
@@ -752,12 +1010,12 @@ def print_executables(
     # search executables by operative system
     if operative_system == "linux":
         blender_executable_filepath = os.path.join(
-            extracted_directory_filepath,
+            extracted_directory_path,
             "blender",
         )
 
         python_executable_filepath = None
-        for dirpath, dirnames, files in os.walk(extracted_directory_filepath):
+        for dirpath, dirnames, files in os.walk(extracted_directory_path):
             if os.path.basename(dirpath) == "bin":
                 for filename in files:
                     if filename.startswith("python"):
@@ -769,12 +1027,12 @@ def print_executables(
                 break
     elif operative_system == "windows":
         blender_executable_filepath = os.path.join(
-            extracted_directory_filepath,
+            extracted_directory_path,
             "blender.exe",
         )
 
         python_executable_filepath = None
-        for dirpath, dirnames, files in os.walk(extracted_directory_filepath):
+        for dirpath, dirnames, files in os.walk(extracted_directory_path):
             if os.path.basename(dirpath) == "bin":
                 for filename in files:
                     if os.path.splitext(filename)[1] == ".exe":
@@ -787,7 +1045,7 @@ def print_executables(
                     break
     else:  # operative_system == "macos"
         python_executable_filepath, blender_executable_filepath = (None, None)
-        for dirpath, dirnames, files in os.walk(extracted_directory_filepath):
+        for dirpath, dirnames, files in os.walk(extracted_directory_path):
             dirname = os.path.basename(dirpath)
             if dirname == "bin":
                 for filename in files:
@@ -883,13 +1141,13 @@ def list_available_blender_versions(
     if maximum_versions < 3:
         sys.stdout.write(f"{stable_version}\n")
         return 0
-    stable_Version = Version(stable_version)
+    stable_Version = BlenderVersion(stable_version)
     _stable_version_printed = False
 
     url = "https://download.blender.org/release/"
     response_lines = GET(url, use_cache=use_cache).splitlines()
     version_matcher = re.compile(r"^\d+\.\d+$")
-    min_blender_Version_supported = Version(MINIMUM_VERSION_SUPPPORTED)
+    min_blender_Version_supported = BlenderVersion(MINIMUM_VERSION_SUPPPORTED)
 
     for line in reversed(response_lines):
         quote_split = line.split('"')
@@ -904,7 +1162,7 @@ def list_available_blender_versions(
         if not re.match(version_matcher, blender_version):
             continue
 
-        major_minor_blender_Version = Version(blender_version)
+        major_minor_blender_Version = BlenderVersion(blender_version)
         if major_minor_blender_Version < min_blender_Version_supported:
             continue
 
@@ -927,6 +1185,8 @@ def list_available_blender_versions(
             major_minor_blender_release_url,
             use_cache=use_cache,
         ).splitlines()
+
+        repo_versions = []
         for repo_line in reversed(repo_response):
             if not repo_line.startswith(f'<a href="blender-{blender_version}'):
                 continue
@@ -938,8 +1198,13 @@ def list_available_blender_versions(
             if not valid_release_file(filename):
                 continue
 
+            repo_versions.append(repo_line.split("-")[1])
+
+        # sort versions in reversed order
+        repo_versions.sort(key=lambda v: BlenderVersion(v), reverse=True)
+
+        for version in repo_versions:
             # found version
-            version = repo_line.split("-")[1]
             if version.count(".") > 2:
                 version = ".".join(version.split(".")[:3])
             if version in versions_found:
@@ -947,7 +1212,7 @@ def list_available_blender_versions(
 
             # print version
             #   print stable version in their correct place
-            if Version(version) < stable_Version and not _stable_version_printed:
+            if BlenderVersion(version) < stable_Version and not _stable_version_printed:
                 n_versions += 1
                 sys.stdout.write(f"{stable_version}\n")
                 _stable_version_printed = True
@@ -977,7 +1242,7 @@ def run(args=[]):
         except CacheTimeout:
             sys.stderr.write(
                 "An error happen clearing blender-downloader's cache.\n"
-                f"Please, submit an issue to {SCRIPT_NEW_ISSUE_URL} if the"
+                f"Please, submit a report to {SCRIPT_NEW_ISSUE_URL} if the"
                 " problem persists.\n"
             )
             return 1
@@ -986,13 +1251,16 @@ def run(args=[]):
             return 0
 
     if opts.list:
-        return list_available_blender_versions(
-            opts.list,
-            opts.operative_system,
-            opts.bits,
-            opts.arch,
-            use_cache=opts.use_cache,
-        )
+        try:
+            return list_available_blender_versions(
+                opts.list,
+                opts.operative_system,
+                opts.bits,
+                opts.arch,
+                use_cache=opts.use_cache,
+            )
+        except KeyboardInterrupt:
+            return 1
     try:
         download_url = get_legacy_release_download_url(
             opts.blender_version,
@@ -1019,33 +1287,49 @@ def run(args=[]):
                 f" {SCRIPT_NEW_ISSUE_URL}\n"
             )
             sys.stderr.write(version_not_found_error_message)
-            sys.exit(1)
-    downloaded_release_filepath = download_release(
-        download_url,
-        opts.output_directory,
-        quiet=QUIET,
-    )
-    if opts.extract:
-        extracted_directory_filepath = extract_release(
-            downloaded_release_filepath,
+            return 1
+        except KeyboardInterrupt:
+            return 1
+    except KeyboardInterrupt:
+        return 1
+
+    downloaded_release_filepath = None
+    try:
+        downloaded_release_filepath = download_release(
+            download_url,
+            opts.output_directory,
             quiet=QUIET,
         )
-
-        if opts.print_blender_executable or opts.print_python_executable:
-            print_executables(
-                extracted_directory_filepath,
-                opts.operative_system,
-                opts.print_blender_executable,
-                opts.print_python_executable,
+        if opts.extract:
+            extracted_directory_path = extract_release(
+                downloaded_release_filepath,
+                quiet=QUIET,
             )
 
-        if opts.remove_compressed:
+            if opts.print_blender_executable or opts.print_python_executable:
+                print_executables(
+                    extracted_directory_path,
+                    opts.operative_system,
+                    opts.print_blender_executable,
+                    opts.print_python_executable,
+                )
+
+            if opts.remove_compressed:
+                os.remove(downloaded_release_filepath)
+    except KeyboardInterrupt:
+        # other keyboard interruption signals are handled inside the
+        # functions, which perform the corresponding cleaning in each
+
+        # clean downloaded file if we were not removed it
+        if opts.remove_compressed and os.path.isfile(downloaded_release_filepath):
             os.remove(downloaded_release_filepath)
 
+        return 1
+
     return 0
 
 
-def main():
+def main():  # pragma: no cover
     sys.exit(run(args=sys.argv[1:]))
 
 
